/* 
 * $smu-mark$ 
 * $name: release.h$
 * $author: Salvatore Sanfilippo <antirez@invece.org>$
 * $copyright: Copyright (C) 1999 by Salvatore Sanfilippo$
 * $license: This software is under GPL version 2 of license$
 * $date: Fri Nov  16 11:55:49 MET 1999$
 * $rev: 17$
 */ 

#ifndef _RELEASE_H
#define _RELEASE_H

#define RELEASE_VERSION "3.0.0-alpha-1"
#define RELEASE_DATE "$Id: release.h,v 1.4 2004/04/09 23:38:56 antirez Exp $"
#define CONTACTS "<antirez@invece.org>"

#endif /* _RELEASE_H */
